@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2021, University of Cincinnati, developed by Henry Schreiner
+// Copyright (c) 2017-2024, University of Cincinnati, developed by Henry Schreiner
 // under NSF AWARD 1414736 and by the respective contributors.
 // All rights reserved.
 //
@@ -22,8 +22,8 @@ template <class T = int> struct Values {
 using DoubleValues = Values<double>;
 
 // the lexical cast operator should be in the same namespace as the type for ADL to work properly
-bool lexical_cast(const std::string &input, Values<double> &v) {
-    std::cout << "called correct lexical_cast function ! val: " << input << std::endl;
+bool lexical_cast(const std::string &input, Values<double> & /*v*/) {
+    std::cout << "called correct lexical_cast function ! val: " << input << '\n';
     return true;
 }
 
