@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2021, University of Cincinnati, developed by Henry Schreiner
+// Copyright (c) 2017-2024, University of Cincinnati, developed by Henry Schreiner
 // under NSF AWARD 1414736 and by the respective contributors.
 // All rights reserved.
 //
@@ -24,14 +24,14 @@ int main(int argc, char **argv) {
 
     app.add_flag("--flag", "This is a flag");
 
-    auto sub1 = app.add_subcommand("one", "Description One");
+    auto *sub1 = app.add_subcommand("one", "Description One");
     sub1->add_flag("--oneflag", "Some flag");
-    auto sub2 = app.add_subcommand("two", "Description Two");
+    auto *sub2 = app.add_subcommand("two", "Description Two");
     sub2->add_flag("--twoflag", "Some other flag");
 
     CLI11_PARSE(app, argc, argv);
 
-    std::cout << "This app was meant to show off the formatter, run with -h" << std::endl;
+    std::cout << "This app was meant to show off the formatter, run with -h" << '\n';
 
     return 0;
 }
