@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2021, University of Cincinnati, developed by Henry Schreiner
+// Copyright (c) 2017-2024, University of Cincinnati, developed by Henry Schreiner
 // under NSF AWARD 1414736 and by the respective contributors.
 // All rights reserved.
 //
@@ -13,6 +13,10 @@
 
 #include "Macros.hpp"
 
+#include "Encoding.hpp"
+
+#include "Argv.hpp"
+
 #include "StringTools.hpp"
 
 #include "Error.hpp"
