@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2021, University of Cincinnati, developed by Henry Schreiner
+// Copyright (c) 2017-2024, University of Cincinnati, developed by Henry Schreiner
 // under NSF AWARD 1414736 and by the respective contributors.
 // All rights reserved.
 //
@@ -10,11 +10,11 @@
 #include "CLI/CLI.hpp"
 #endif
 
+#include "app_helper.hpp"
+
 #include "catch.hpp"
 #include <fstream>
 
-using Catch::Matchers::Contains;
-
 TEST_CASE("THelp: Basic", "[help]") {
     CLI::App app{"My prog"};
 
@@ -26,6 +26,43 @@ TEST_CASE("THelp: Basic", "[help]") {
     CHECK_THAT(help, Contains("Usage:"));
 }
 
+TEST_CASE("THelp: Usage", "[help]") {
+    CLI::App app{"My prog"};
+    app.usage("use: just use it");
+
+    std::string help = app.help();
+
+    CHECK_THAT(help, Contains("My prog"));
+    CHECK_THAT(help, Contains("-h,--help"));
+    CHECK_THAT(help, Contains("Options:"));
+    CHECK_THAT(help, Contains("use: just use it"));
+}
+
+TEST_CASE("THelp: UsageCallback", "[help]") {
+    CLI::App app{"My prog"};
+    app.usage([]() { return "use: just use it"; });
+
+    std::string help = app.help();
+
+    CHECK_THAT(help, Contains("My prog"));
+    CHECK_THAT(help, Contains("-h,--help"));
+    CHECK_THAT(help, Contains("Options:"));
+    CHECK_THAT(help, Contains("use: just use it"));
+}
+
+TEST_CASE("THelp: UsageCallbackBoth", "[help]") {
+    CLI::App app{"My prog"};
+    app.usage([]() { return "use: just use it"; });
+    app.usage("like 1, 2, and 3");
+    std::string help = app.help();
+
+    CHECK_THAT(help, Contains("My prog"));
+    CHECK_THAT(help, Contains("-h,--help"));
+    CHECK_THAT(help, Contains("Options:"));
+    CHECK_THAT(help, Contains("use: just use it"));
+    CHECK_THAT(help, Contains("like 1, 2, and 3"));
+}
+
 TEST_CASE("THelp: Footer", "[help]") {
     CLI::App app{"My prog"};
     app.footer("Report bugs to bugs@example.com");
@@ -105,7 +142,7 @@ TEST_CASE("THelp: deprecatedOptions", "[help]") {
     CLI::App app{"My prog"};
 
     std::string x;
-    auto soption = app.add_option("--something", x, "My option here");
+    auto *soption = app.add_option("--something", x, "My option here");
     app.add_option("--something_else", x, "My option here");
     std::string y;
     app.add_option("--another", y);
@@ -159,7 +196,7 @@ TEST_CASE("THelp: retiredOptions", "[help]") {
     CLI::App app{"My prog"};
 
     std::string x;
-    auto opt1 = app.add_option("--something", x, "My option here");
+    auto *opt1 = app.add_option("--something", x, "My option here");
     app.add_option("--something_else", x, "My option here");
     std::string y;
     app.add_option("--another", y);
@@ -213,7 +250,7 @@ TEST_CASE("THelp: retiredOptions3", "[help]") {
 TEST_CASE("THelp: HiddenGroup", "[help]") {
     CLI::App app{"My prog"};
     // empty option group name should be hidden
-    auto hgroup = app.add_option_group("");
+    auto *hgroup = app.add_option_group("");
     std::string x;
     hgroup->add_option("something", x, "My option here");
     std::string y;
@@ -289,7 +326,8 @@ TEST_CASE("THelp: VectorOpts", "[help]") {
 
     std::string help = app.help();
 
-    CHECK_THAT(help, Contains("INT=[1,2] ..."));
+    CHECK_THAT(help, Contains("[1,2]"));
+    CHECK_THAT(help, Contains(" ..."));
 }
 
 TEST_CASE("THelp: MultiPosOpts", "[help]") {
@@ -394,18 +432,18 @@ TEST_CASE("THelp: ManualSetters", "[help]") {
 
     std::string help = app.help();
 
-    CHECK_THAT(help, Contains("=12"));
+    CHECK_THAT(help, Contains("[12]"));
     CHECK_THAT(help, Contains("BIGGLES"));
 
     op1->default_val("14");
     CHECK(14 == x);
     help = app.help();
-    CHECK_THAT(help, Contains("=14"));
+    CHECK_THAT(help, Contains("[14]"));
 
     op1->default_val(12);
     CHECK(12 == x);
     help = app.help();
-    CHECK_THAT(help, Contains("=12"));
+    CHECK_THAT(help, Contains("[12]"));
 
     CHECK(op1->get_run_callback_for_default());
     op1->run_callback_for_default(false);
@@ -415,7 +453,7 @@ TEST_CASE("THelp: ManualSetters", "[help]") {
     // x should not be modified in this case
     CHECK(12 == x);
     help = app.help();
-    CHECK_THAT(help, Contains("=18"));
+    CHECK_THAT(help, Contains("[18]"));
 }
 
 TEST_CASE("THelp: ManualSetterOverFunction", "[help]") {
@@ -432,7 +470,7 @@ TEST_CASE("THelp: ManualSetterOverFunction", "[help]") {
     CHECK(1 == x);
 
     std::string help = app.help();
-    CHECK_THAT(help, Contains("=12"));
+    CHECK_THAT(help, Contains("[12]"));
     CHECK_THAT(help, Contains("BIGGLES"));
     CHECK_THAT(help, Contains("QUIGGLES"));
     CHECK_THAT(help, Contains("{1,2}"));
@@ -441,7 +479,7 @@ TEST_CASE("THelp: ManualSetterOverFunction", "[help]") {
 TEST_CASE("THelp: Subcom", "[help]") {
     CLI::App app{"My prog"};
 
-    auto sub1 = app.add_subcommand("sub1");
+    auto *sub1 = app.add_subcommand("sub1");
     app.add_subcommand("sub2");
 
     std::string help = app.help();
@@ -455,8 +493,8 @@ TEST_CASE("THelp: Subcom", "[help]") {
     help = sub1->help();
     CHECK_THAT(help, Contains("Usage: sub1"));
 
-    char x[] = "./myprogram";
-    char y[] = "sub2";
+    char x[] = "./myprogram";  // NOLINT(modernize-avoid-c-arrays)
+    char y[] = "sub2";         // NOLINT(modernize-avoid-c-arrays)
 
     std::vector<char *> args = {x, y};
     app.parse(static_cast<int>(args.size()), args.data());
@@ -468,7 +506,7 @@ TEST_CASE("THelp: Subcom", "[help]") {
 TEST_CASE("THelp: Subcom_alias", "[help]") {
     CLI::App app{"My prog"};
 
-    auto sub1 = app.add_subcommand("sub1", "Subcommand1 description test");
+    auto *sub1 = app.add_subcommand("sub1", "Subcommand1 description test");
     sub1->alias("sub_alias1");
     sub1->alias("sub_alias2");
 
@@ -483,7 +521,7 @@ TEST_CASE("THelp: Subcom_alias", "[help]") {
 TEST_CASE("THelp: Subcom_alias_group", "[help]") {
     CLI::App app{"My prog"};
 
-    auto sub1 = app.add_subcommand("", "Subcommand1 description test");
+    auto *sub1 = app.add_subcommand("", "Subcommand1 description test");
     sub1->alias("sub_alias1");
     sub1->alias("sub_alias2");
 
@@ -498,7 +536,7 @@ TEST_CASE("THelp: Subcom_alias_group", "[help]") {
 TEST_CASE("THelp: MasterName", "[help]") {
     CLI::App app{"My prog", "MyRealName"};
 
-    char x[] = "./myprogram";
+    char x[] = "./myprogram";  // NOLINT(modernize-avoid-c-arrays)
 
     std::vector<char *> args = {x};
     app.parse(static_cast<int>(args.size()), args.data());
@@ -518,7 +556,7 @@ TEST_CASE("THelp: IntDefaults", "[help]") {
     CHECK_THAT(help, Contains("--one"));
     CHECK_THAT(help, Contains("--set"));
     CHECK_THAT(help, Contains("1"));
-    CHECK_THAT(help, Contains("=2"));
+    CHECK_THAT(help, Contains("[2]"));
     CHECK_THAT(help, Contains("2,3,4"));
 }
 
@@ -532,7 +570,7 @@ TEST_CASE("THelp: SetLower", "[help]") {
     std::string help = app.help();
 
     CHECK_THAT(help, Contains("--set"));
-    CHECK_THAT(help, Contains("=One"));
+    CHECK_THAT(help, Contains("[One]"));
     CHECK_THAT(help, Contains("oNe"));
     CHECK_THAT(help, Contains("twO"));
     CHECK_THAT(help, Contains("THREE"));
@@ -682,6 +720,22 @@ TEST_CASE("THelp: CustomHelp", "[help]") {
     }
 }
 
+TEST_CASE("THelp: HelpSubcommandPriority", "[help]") {
+    CLI::App app{"My prog"};
+
+    app.set_help_flag("-h", "display help and exit");
+
+    auto *sub1 = app.add_subcommand("sub1");
+    std::string someFile = "";
+
+    put_env("SOME_FILE", "NOT_A_FILE");
+    sub1->add_option("-f,--file", someFile)->envname("SOME_FILE")->required()->expected(1)->check(CLI::ExistingFile);
+
+    std::string input{"sub1 -h"};
+    CHECK_THROWS_AS(app.parse(input), CLI::CallForHelp);
+    unset_env("SOME_FILE");
+}
+
 TEST_CASE("THelp: NextLineShouldBeAlignmentInMultilineDescription", "[help]") {
     CLI::App app;
     int i{0};
@@ -698,9 +752,9 @@ TEST_CASE("THelp: NiceName", "[help]") {
     CLI::App app;
 
     int x{0};
-    auto long_name = app.add_option("-s,--long,-q,--other,that", x);
-    auto short_name = app.add_option("more,-x,-y", x);
-    auto positional = app.add_option("posit", x);
+    auto *long_name = app.add_option("-s,--long,-q,--other,that", x);
+    auto *short_name = app.add_option("more,-x,-y", x);
+    auto *positional = app.add_option("posit", x);
 
     CHECK("--long" == long_name->get_name());
     CHECK("-x" == short_name->get_name());
@@ -767,25 +821,25 @@ struct CapturedHelp {
 
 TEST_CASE_METHOD(CapturedHelp, "Successful", "[help]") {
     CHECK(0 == run(CLI::Success()));
-    CHECK("" == out.str());
-    CHECK("" == err.str());
+    CHECK(out.str().empty());
+    CHECK(err.str().empty());
 }
 
 TEST_CASE_METHOD(CapturedHelp, "JustAnError", "[help]") {
     CHECK(42 == run(CLI::RuntimeError(42)));
-    CHECK("" == out.str());
-    CHECK("" == err.str());
+    CHECK(out.str().empty());
+    CHECK(err.str().empty());
 }
 
 TEST_CASE_METHOD(CapturedHelp, "CallForHelp", "[help]") {
     CHECK(0 == run(CLI::CallForHelp()));
     CHECK(app.help() == out.str());
-    CHECK("" == err.str());
+    CHECK(err.str().empty());
 }
 TEST_CASE_METHOD(CapturedHelp, "CallForAllHelp", "[help]") {
     CHECK(0 == run(CLI::CallForAllHelp()));
     CHECK(app.help("", CLI::AppFormatMode::All) == out.str());
-    CHECK("" == err.str());
+    CHECK(err.str().empty());
 }
 TEST_CASE_METHOD(CapturedHelp, "CallForAllHelpOutput", "[help]") {
     app.set_help_all_flag("--help-all", "Help all");
@@ -795,7 +849,7 @@ TEST_CASE_METHOD(CapturedHelp, "CallForAllHelpOutput", "[help]") {
 
     CHECK(0 == run(CLI::CallForAllHelp()));
     CHECK(app.help("", CLI::AppFormatMode::All) == out.str());
-    CHECK("" == err.str());
+    CHECK(err.str().empty());
     CHECK_THAT(out.str(), Contains("one"));
     CHECK_THAT(out.str(), Contains("two"));
     CHECK_THAT(out.str(), Contains("--three"));
@@ -812,18 +866,18 @@ TEST_CASE_METHOD(CapturedHelp, "CallForAllHelpOutput", "[help]") {
                        "  One description\n\n"
                        "two\n"
                        "  Options:\n"
-                       "    --three                     \n\n\n");
+                       "    --three                     \n\n");
 }
 TEST_CASE_METHOD(CapturedHelp, "NewFormattedHelp", "[help]") {
     app.formatter_fn([](const CLI::App *, std::string, CLI::AppFormatMode) { return "New Help"; });
     CHECK(0 == run(CLI::CallForHelp()));
     CHECK("New Help" == out.str());
-    CHECK("" == err.str());
+    CHECK(err.str().empty());
 }
 
 TEST_CASE_METHOD(CapturedHelp, "NormalError", "[help]") {
     CHECK(static_cast<int>(CLI::ExitCodes::ExtrasError) == run(CLI::ExtrasError({"Thing"})));
-    CHECK("" == out.str());
+    CHECK(out.str().empty());
     CHECK_THAT(err.str(), Contains("for more information"));
     CHECK_THAT(err.str(), !Contains("ExtrasError"));
     CHECK_THAT(err.str(), Contains("Thing"));
@@ -834,7 +888,7 @@ TEST_CASE_METHOD(CapturedHelp, "NormalError", "[help]") {
 TEST_CASE_METHOD(CapturedHelp, "DoubleError", "[help]") {
     app.set_help_all_flag("--help-all");
     CHECK(static_cast<int>(CLI::ExitCodes::ExtrasError) == run(CLI::ExtrasError({"Thing"})));
-    CHECK("" == out.str());
+    CHECK(out.str().empty());
     CHECK_THAT(err.str(), Contains("for more information"));
     CHECK_THAT(err.str(), Contains(" --help "));
     CHECK_THAT(err.str(), Contains(" --help-all "));
@@ -848,7 +902,7 @@ TEST_CASE_METHOD(CapturedHelp, "AllOnlyError", "[help]") {
     app.set_help_all_flag("--help-all");
     app.set_help_flag();
     CHECK(static_cast<int>(CLI::ExitCodes::ExtrasError) == run(CLI::ExtrasError({"Thing"})));
-    CHECK("" == out.str());
+    CHECK(out.str().empty());
     CHECK_THAT(err.str(), Contains("for more information"));
     CHECK_THAT(err.str(), !Contains(" --help "));
     CHECK_THAT(err.str(), Contains(" --help-all "));
@@ -862,7 +916,7 @@ TEST_CASE_METHOD(CapturedHelp, "ReplacedError", "[help]") {
     app.failure_message(CLI::FailureMessage::help);
 
     CHECK(static_cast<int>(CLI::ExitCodes::ExtrasError) == run(CLI::ExtrasError({"Thing"})));
-    CHECK("" == out.str());
+    CHECK(out.str().empty());
     CHECK_THAT(err.str(), !Contains("for more information"));
     CHECK_THAT(err.str(), Contains("ERROR: ExtrasError"));
     CHECK_THAT(err.str(), Contains("Thing"));
@@ -876,7 +930,7 @@ TEST_CASE("THelp: CustomDoubleOption", "[help]") {
 
     CLI::App app;
 
-    auto opt = app.add_option("posit", [&custom_opt](CLI::results_t vals) {
+    auto *opt = app.add_option("posit", [&custom_opt](CLI::results_t vals) {
         custom_opt = {stol(vals.at(0)), stod(vals.at(1))};
         return true;
     });
@@ -888,11 +942,19 @@ TEST_CASE("THelp: CustomDoubleOption", "[help]") {
 TEST_CASE("THelp: CheckEmptyTypeName", "[help]") {
     CLI::App app;
 
-    auto opt = app.add_flag("-f,--flag");
+    auto *opt = app.add_flag("-f,--flag");
     std::string name = opt->get_type_name();
     CHECK(name.empty());
 }
 
+TEST_CASE("THelp: FlagDefaults", "[help]") {
+    CLI::App app;
+
+    app.add_flag("-t,--not{false}")->default_str("false");
+    auto str = app.help();
+    CHECK_THAT(str, Contains("--not{false}"));
+}
+
 TEST_CASE("THelp: AccessDescription", "[help]") {
     CLI::App app{"My description goes here"};
 
@@ -912,7 +974,7 @@ TEST_CASE("THelp: AccessOptionDescription", "[help]") {
     CLI::App app{};
 
     int x{0};
-    auto opt = app.add_option("-a,--alpha", x, "My description goes here");
+    auto *opt = app.add_option("-a,--alpha", x, "My description goes here");
 
     CHECK("My description goes here" == opt->get_description());
 }
@@ -921,7 +983,7 @@ TEST_CASE("THelp: SetOptionDescriptionAfterCreation", "[help]") {
     CLI::App app{};
 
     int x{0};
-    auto opt = app.add_option("-a,--alpha", x);
+    auto *opt = app.add_option("-a,--alpha", x);
     opt->description("My description goes here");
 
     CHECK("My description goes here" == opt->get_description());
@@ -932,7 +994,7 @@ TEST_CASE("THelp: CleanNeeds", "[help]") {
     CLI::App app;
 
     int x{0};
-    auto a_name = app.add_option("-a,--alpha", x);
+    auto *a_name = app.add_option("-a,--alpha", x);
     app.add_option("-b,--boo", x)->needs(a_name);
 
     CHECK_THAT(app.help(), !Contains("Requires"));
@@ -965,6 +1027,16 @@ TEST_CASE("THelp: GroupOrder", "[help]") {
     CHECK(aee_loc > zee_loc);
 }
 
+TEST_CASE("THelp: GroupNameError", "[help]") {
+    CLI::App app;
+
+    auto *f1 = app.add_flag("--one");
+    auto *f2 = app.add_flag("--two");
+
+    CHECK_THROWS_AS(f1->group("evil group name\non two lines"), CLI::IncorrectConstruction);
+    CHECK_THROWS_AS(f2->group(std::string(5, '\0')), CLI::IncorrectConstruction);
+}
+
 TEST_CASE("THelp: ValidatorsText", "[help]") {
     CLI::App app;
 
@@ -1162,7 +1234,9 @@ TEST_CASE("THelp: ChangingDefaults", "[help]") {
     x = {5, 6};
     std::string help = app.help();
 
-    CHECK_THAT(help, Contains("INT=[3,4] ..."));
+    CHECK_THAT(help, Contains("[[3,4]]"));
+    CHECK_THAT(help, Contains("..."));
+    CHECK_THAT(help, Contains("INT"));
     CHECK(x[0] == 5);
 }
 
@@ -1179,7 +1253,8 @@ TEST_CASE("THelp: ChangingDefaultsWithAutoCapture", "[help]") {
 
     std::string help = app.help();
 
-    CHECK_THAT(help, Contains("INT=[1,2] ..."));
+    CHECK_THAT(help, Contains("[[1,2]]"));
+    CHECK_THAT(help, Contains("..."));
 }
 
 TEST_CASE("THelp: FunctionDefaultString", "[help]") {
@@ -1194,7 +1269,7 @@ TEST_CASE("THelp: FunctionDefaultString", "[help]") {
 
     std::string help = app.help();
 
-    CHECK_THAT(help, Contains("INT=Powerful"));
+    CHECK_THAT(help, Contains("[Powerful]"));
 }
 
 TEST_CASE("TVersion: simple_flag", "[help]") {
@@ -1248,16 +1323,32 @@ TEST_CASE("TVersion: parse_throw", "[help]") {
     CHECK_THROWS_AS(app.parse("--version"), CLI::CallForVersion);
     CHECK_THROWS_AS(app.parse("--version --arg2 5"), CLI::CallForVersion);
 
-    auto ptr = app.get_version_ptr();
+    auto *ptr = app.get_version_ptr();
 
     ptr->ignore_case();
     try {
         app.parse("--Version");
     } catch(const CLI::CallForVersion &v) {
-        CHECK_THAT(CLI11_VERSION, Catch::Equals(v.what()));
+        CHECK_THAT(CLI11_VERSION, Equals(v.what()));
         CHECK(0 == v.get_exit_code());
         const auto &appc = app;
-        auto cptr = appc.get_version_ptr();
+        const auto *cptr = appc.get_version_ptr();
         CHECK(1U == cptr->count());
     }
 }
+
+TEST_CASE("TVersion: exit", "[help]") {
+
+    CLI::App app;
+
+    app.set_version_flag("--version", CLI11_VERSION);
+
+    try {
+        app.parse("--version");
+    } catch(const CLI::CallForVersion &v) {
+        std::ostringstream out;
+        auto ret = app.exit(v, out);
+        CHECK_THAT(out.str(), Contains(CLI11_VERSION));
+        CHECK(0 == ret);
+    }
+}
